# WaveShare 0.96" 160x80 pixel lcd Workout
# Tony Goodhew 1 November 2021
# Display driver from:
# https://www.waveshare.com/wiki/Pico-lcd-0.96
# Added re-sizeable Fonts and colour mixing
from machine import Pin,SPI,PWM
import framebuf
import utime
import math
import random

# Setup Joystick and buttons = Zero if closed
KEY_UP = Pin(2,Pin.IN,Pin.PULL_UP)
KEY_DOWN = Pin(18,Pin.IN,Pin.PULL_UP)
KEY_LEFT= Pin(16,Pin.IN,Pin.PULL_UP)
KEY_RIGHT= Pin(20,Pin.IN,Pin.PULL_UP)
KEY_CTRL=Pin(3,Pin.IN,Pin.PULL_UP)
KEY_A=Pin(15,Pin.IN,Pin.PULL_UP)
KEY_B=Pin(17,Pin.IN,Pin.PULL_UP)

RED = 0x00F8
GREEN = 0xE007
BLUE = 0x1F00
WHITE = 0xFFFF
BLACK = 0x0000
YELLOW = GREEN + RED
MAGENTA = RED + BLUE
CYAN = BLUE + GREEN

class lcd_0inch96(framebuf.FrameBuffer): # WaveShare supplied screen driver
    def __init__(self):
        
        self.width = 160 # Pixel dimensions of screen
        self.height = 80
        
        self.cs = Pin(9,Pin.OUT)
        self.rst = Pin(12,Pin.OUT)
#        self.bl = Pin(13,Pin.OUT)
        self.cs(1)
        # pwm = PWM(Pin(13))#BL
        # pwm.freq(1000)        
        self.spi = SPI(1)
        self.spi = SPI(1,1000_000)
        self.spi = SPI(1,10000_000,polarity=0, phase=0,sck=Pin(10),mosi=Pin(11),miso=None)
        self.dc = Pin(8,Pin.OUT)
        self.dc(1)
        self.buffer = bytearray(self.height * self.width * 2)
        super().__init__(self.buffer, self.width, self.height, framebuf.RGB565)
        self.Init()
        self.SetWindows(0, 0, self.width-1, self.height-1)
        
    def reset(self):
        self.rst(1)
        utime.sleep(0.2) 
        self.rst(0)
        utime.sleep(0.2)         
        self.rst(1)
        utime.sleep(0.2) 
        
    def write_cmd(self, cmd):
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))

    def write_data(self, buf):
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)

    def backlight(self,value):#value:  min:0  max:1000
        pwm = PWM(Pin(13))#BL
        pwm.freq(1000)
        if value>=1000:
            value=1000
        data=int (value*65536/1000)       
        pwm.duty_u16(data)  
        
    def Init(self):
        self.reset() 
        self.backlight(10000)  
        
        self.write_cmd(0x11)
        utime.sleep(0.12)
        self.write_cmd(0x21) 
        self.write_cmd(0x21) 

        self.write_cmd(0xB1) 
        self.write_data(0x05)
        self.write_data(0x3A)
        self.write_data(0x3A)

        self.write_cmd(0xB2)
        self.write_data(0x05)
        self.write_data(0x3A)
        self.write_data(0x3A)

        self.write_cmd(0xB3) 
        self.write_data(0x05)  
        self.write_data(0x3A)
        self.write_data(0x3A)
        self.write_data(0x05)
        self.write_data(0x3A)
        self.write_data(0x3A)

        self.write_cmd(0xB4)
        self.write_data(0x03)

        self.write_cmd(0xC0)
        self.write_data(0x62)
        self.write_data(0x02)
        self.write_data(0x04)

        self.write_cmd(0xC1)
        self.write_data(0xC0)

        self.write_cmd(0xC2)
        self.write_data(0x0D)
        self.write_data(0x00)

        self.write_cmd(0xC3)
        self.write_data(0x8D)
        self.write_data(0x6A)   

        self.write_cmd(0xC4)
        self.write_data(0x8D) 
        self.write_data(0xEE) 

        self.write_cmd(0xC5)
        self.write_data(0x0E)    

        self.write_cmd(0xE0)
        self.write_data(0x10)
        self.write_data(0x0E)
        self.write_data(0x02)
        self.write_data(0x03)
        self.write_data(0x0E)
        self.write_data(0x07)
        self.write_data(0x02)
        self.write_data(0x07)
        self.write_data(0x0A)
        self.write_data(0x12)
        self.write_data(0x27)
        self.write_data(0x37)
        self.write_data(0x00)
        self.write_data(0x0D)
        self.write_data(0x0E)
        self.write_data(0x10)

        self.write_cmd(0xE1)
        self.write_data(0x10)
        self.write_data(0x0E)
        self.write_data(0x03)
        self.write_data(0x03)
        self.write_data(0x0F)
        self.write_data(0x06)
        self.write_data(0x02)
        self.write_data(0x08)
        self.write_data(0x0A)
        self.write_data(0x13)
        self.write_data(0x26)
        self.write_data(0x36)
        self.write_data(0x00)
        self.write_data(0x0D)
        self.write_data(0x0E)
        self.write_data(0x10)

        self.write_cmd(0x3A) 
        self.write_data(0x05)

        self.write_cmd(0x36)
        self.write_data(0xA8)

        self.write_cmd(0x29) 
        
    def SetWindows(self, Xstart, Ystart, Xend, Yend):#example max:0,0,159,79
        Xstart=Xstart+1
        Xend=Xend+1
        Ystart=Ystart+26
        Yend=Yend+26
        self.write_cmd(0x2A)
        self.write_data(0x00)              
        self.write_data(Xstart)      
        self.write_data(0x00)              
        self.write_data(Xend) 

        self.write_cmd(0x2B)
        self.write_data(0x00)
        self.write_data(Ystart)
        self.write_data(0x00)
        self.write_data(Yend)

        self.write_cmd(0x2C) 
        
    def display(self):
    
        self.SetWindows(0,0,self.width-1,self.height-1)       
        self.dc(1)
        self.cs(0)
        self.spi.write(self.buffer)
        self.cs(1)        
'''
Adjustable font for the WaveShare 0.96" IPS lcd Display Module for Raspberry Pi Pico (160x80)
                         Tony Goodhew 1 Nov 2021
           Modified from code by Les Wright 2021 V 1.1 for Pimoroni Pico Display                  
              https://forums.pimoroni.com/t/pico-display-and-fonts/16194/18
'''
#ASCII Character Set
cmap = ['00000000000000000000000000000000000', #Space
        '00100001000010000100001000000000100', #!
        '01010010100000000000000000000000000', #"
        '01010010101101100000110110101001010', ##
        '00100011111000001110000011111000100', #$
        '11001110010001000100010001001110011', #%
        '01000101001010001000101011001001101', #&
        '10000100001000000000000000000000000', #'
        '00100010001000010000100000100000100', #(
        '00100000100000100001000010001000100', #)
        '00000001001010101110101010010000000', #*
        '00000001000010011111001000010000000', #+
        '000000000000000000000000000000110000100010000', #,
        '00000000000000011111000000000000000', #-
        '00000000000000000000000001100011000', #.
        '00001000010001000100010001000010000', #/
        '01110100011000110101100011000101110', #0
        '00100011000010000100001000010001110', #1
        '01110100010000101110100001000011111', #2
        '01110100010000101110000011000101110', #3
        '00010001100101011111000100001000010', #4
        '11111100001111000001000011000101110', #5
        '01110100001000011110100011000101110', #6
        '11111000010001000100010001000010000', #7
        '01110100011000101110100011000101110', #8
        '01110100011000101111000010000101110', #9
        '00000011000110000000011000110000000', #:
        '01100011000000001100011000010001000', #;
        '00010001000100010000010000010000010', #<
        '00000000001111100000111110000000000', #=
        '01000001000001000001000100010001000', #>
        '01100100100001000100001000000000100', #?
        '01110100010000101101101011010101110', #@
        '00100010101000110001111111000110001', #A
        '11110010010100111110010010100111110', #B
        '01110100011000010000100001000101110', #C
        '11110010010100101001010010100111110', #D
        '11111100001000011100100001000011111', #E
        '11111100001000011100100001000010000', #F
        '01110100011000010111100011000101110', #G
        '10001100011000111111100011000110001', #H
        '01110001000010000100001000010001110', #I
        '00111000100001000010000101001001100', #J
        '10001100101010011000101001001010001', #K
        '10000100001000010000100001000011111', #L
        '10001110111010110101100011000110001', #M
        '10001110011010110011100011000110001', #N
        '01110100011000110001100011000101110', #O
        '11110100011000111110100001000010000', #P
        '01110100011000110001101011001001101', #Q
        '11110100011000111110101001001010001', #R
        '01110100011000001110000011000101110', #S
        '11111001000010000100001000010000100', #T
        '10001100011000110001100011000101110', #U
        '10001100011000101010010100010000100', #V
        '10001100011000110101101011101110001', #W
        '10001100010101000100010101000110001', #X
        '10001100010101000100001000010000100', #Y
        '11111000010001000100010001000011111', #Z
        '01110010000100001000010000100001110', #[
        '10000100000100000100000100000100001', #\
        '00111000010000100001000010000100111', #]
        '00100010101000100000000000000000000', #^
        '00000000000000000000000000000011111', #_
        '11000110001000001000000000000000000', #`
        '00000000000111000001011111000101110', #a
        '10000100001011011001100011100110110', #b
        '00000000000011101000010000100000111', #c
        '00001000010110110011100011001101101', #d
        '00000000000111010001111111000001110', #e
        '00110010010100011110010000100001000', #f
        '000000000001110100011000110001011110000101110', #g
        '10000100001011011001100011000110001', #h
        '00100000000110000100001000010001110', #i
        '0001000000001100001000010000101001001100', #j
        '10000100001001010100110001010010010', #k
        '01100001000010000100001000010001110', #l
        '00000000001101010101101011010110101', #m
        '00000000001011011001100011000110001', #n
        '00000000000111010001100011000101110', #o
        '000000000001110100011000110001111101000010000', #p
        '000000000001110100011000110001011110000100001', #q
        '00000000001011011001100001000010000', #r
        '00000000000111110000011100000111110', #s
        '00100001000111100100001000010000111', #t
        '00000000001000110001100011001101101', #u
        '00000000001000110001100010101000100', #v
        '00000000001000110001101011010101010', #w
        '00000000001000101010001000101010001', #x
        '000000000010001100011000110001011110000101110', #y
        '00000000001111100010001000100011111', #z
        '00010001000010001000001000010000010', #{
        '00100001000010000000001000010000100', #|
        '01000001000010000010001000010001000', #}
        '01000101010001000000000000000000000' #}~
]

def printchar(letter,xpos,ypos,size,charupdate,c):
    origin = xpos
    charval = ord(letter)
    #print(charval)
    index = charval-32 #start code, 32 or space
    #print(index)
    character = cmap[index] #this is our char...
    rows = [character[i:i+5] for i in range(0,len(character),5)]
    #print(rows)
    for row in rows:
        #print(row)
        for bit in row:
            #print(bit)
            if bit == '1':
                lcd.pixel(xpos,ypos,c)
                if size==2:
                    lcd.pixel(xpos,ypos+1,c)
                    lcd.pixel(xpos+1,ypos,c)
                    lcd.pixel(xpos+1,ypos+1,c)
                if size == 3:
                    lcd.pixel(xpos,ypos,c)
                    lcd.pixel(xpos,ypos+1,c)
                    lcd.pixel(xpos,ypos+2,c)
                    lcd.pixel(xpos+1,ypos,c)
                    lcd.pixel(xpos+1,ypos+1,c)
                    lcd.pixel(xpos+1,ypos+2,c)
                    lcd.pixel(xpos+2,ypos,c)
                    lcd.pixel(xpos+2,ypos+1,c)
                    lcd.pixel(xpos+2,ypos+2,c)
                                        
            xpos+=size
        xpos=origin
        ypos+=size
    if charupdate == True:
        lcd.display()
    
def delchar(xpos,ypos,size,delupdate):
    if size == 1:
        charwidth = 5
        charheight = 9
    if size == 2:
        charwidth = 10
        charheight = 18
    if size == 3:
        charwidth = 15
        charheight = 27
    c =colour(0,0,0) # Colour of background
    lcd.fill_rect(xpos,ypos,charwidth,charheight,c) #xywh
    if delupdate == True:
        lcd.display()

def printstring(string,xpos,ypos,size,charupdate,strupdate,c):   
    if size == 1:
        spacing = 8
    if size == 2:
        spacing = 14
    if size == 3:
        spacing = 18
    for i in string:
        printchar(i,xpos,ypos,size,charupdate,c)
        xpos+=spacing
    if strupdate == True:
        lcd.display()
        
def centrestring(string,ypos,size,charupdate,strupdate,c):
    if size == 1:
        spacing = 8
    if size == 2:
        spacing = 14
    if size == 3:
        spacing = 18
    used = len(string)*spacing
    xpos = int((159-used)/2)
    printstring(string,xpos,ypos,size,charupdate,strupdate,c)
# =============End of Characters section ===============

# Colour Mixing Routine
def colour(R,G,B): # Compact method!
    mix1 = ((R&0xF8)*256) + ((G&0xFC)*8) + ((B&0xF8)>>3)
    return  (mix1 & 0xFF) *256  + int((mix1 & 0xFF00) /256) # low nibble first

def graphs(): # Draw Sine and Cosine graphs
    lcd.fill(0)
    lcd.display()
    c = colour(80,80,80)
    factor = 361 /160
    lcd.hline(0,40,160,c)    
    lcd.display()
    c = colour(255,0,0)
    for x in range(0,160):
        y = int ((math.sin(math.radians(x * factor)))* -30) + 40
        lcd.pixel(x,y,c)
        lcd.display()
    printstring("Sine", 20, 55, 2, 0,0,c)
    lcd.display()
    utime.sleep(3)
    
    lcd.fill(0)
    lcd.display()
    c = colour(80,80,80)
    lcd.hline(0,40,160,c)    
    lcd.display()
    c = colour(0,255,0)
    for x in range(0,160):
        y = int((math.cos(math.radians(x * factor)))* -30) + 40
        lcd.pixel(x,y,c)
    printstring("Cosine",40,10,2,0,0,c)
    lcd.display()
    utime.sleep(3)
    lcd.fill(0)
    lcd.display()

def bar(): # Dynamic bar graph under joystick control
    lcd.fill(0) # BLACK
    lcd.display()
    c = colour(0,255,0)
    printstring("Bar graph",5,5,1,0,0,c)
    c = colour(160,160,160)
    printstring("Move Joystick",30,54,1,0,0,c)
    printstring("Left and Right",25,63,1,0,0,c)
    printstring("End with button A",15,73,1,0,0,c)
    c = colour(150,150,150)
    lcd.fill_rect(18,20,2,40,c)
    v = 30
    c = colour(255,0,0)
    lcd.fill_rect(20,30,v*2,20,c)
    lcd.display()
    active = True
    while active:
        if(KEY_LEFT.value() == 0):
            v = v - 1
            if v < 0:
                v = 0          
        if(KEY_RIGHT.value() == 0):
            v = v + 1
            if v > 100:
                v = 100
        c = 0
        lcd.fill_rect(20,30,200,20,c) # Blanking old bar       
        c = colour(int(255*v/100),0,255-int(255*v/100)) # Calculate bar colour Blue -> Magenta -> Red
        lcd.fill_rect(20,30,int(v*1.3),20,c) # Draw bar
        c = 0
        lcd.fill_rect(85,10,120,15,c) # Blanking old percentage
        c = colour(255,255,0)
        s = str(v) + "%"
        if (v < 100):
            s =" " + s
        if (v < 10):
            s = "" + s
        printstring(s,85,10,2,0,0,c)
        lcd.display()
        if(KEY_A.value() == 0): # Finished?
            lcd.fill(0) # BLACK
            lcd.display()
            active = False

def circle(x,y,r,c):
    lcd.hline(x-r,y,r*2,c)
    for i in range(1,r):
        a = int(math.sqrt(r*r-i*i)) # Pythagoras!
        lcd.hline(x-a,y+i,a*2,c) # Lower half
        lcd.hline(x-a,y-i,a*2,c) # Upper half

def ring(x,y,r,c):
    lcd.pixel(x-r,y,c)
    lcd.pixel(x+r,y,c)
    lcd.pixel(x,y-r,c)
    lcd.pixel(x,y+r,c)
    for i in range(1,r):
        a = int(math.sqrt(r*r-i*i))
        lcd.pixel(x-a,y-i,c)
        lcd.pixel(x+a,y-i,c)
        lcd.pixel(x-a,y+i,c)
        lcd.pixel(x+a,y+i,c)
        lcd.pixel(x-i,y-a,c)
        lcd.pixel(x+i,y-a,c)
        lcd.pixel(x-i,y+a,c)
        lcd.pixel(x+i,y+a,c)
        
               
# ==== Main ====  
lcd = lcd_0inch96()   # Initialise the display
lcd.fill(BLACK)       # Clear screen

lcd.text(" WaveShare",35,15,colour(255,0,0))     # Red - Built in Font
lcd.text("Display",50,35,colour(0,255,0))        # Green  
lcd.text("Pico-LCD-0.96",30,55,colour(255,255,0)) # Yellow
lcd.display()
# Inner frame
lcd.hline(10,10,140,CYAN)
lcd.hline(10,70,140,CYAN)
lcd.vline(10,10,60,CYAN)
lcd.vline(150,10,60,CYAN)
# Middle Frame
lcd.hline(5,5,150,MAGENTA)
lcd.hline(5,75,150,MAGENTA)
lcd.vline(5,5,70,MAGENTA)
lcd.vline(154,5,70,MAGENTA) 
# Outer Frame - method 1
lcd.hline(0,0,160,BLUE)
lcd.hline(0,79,160,BLUE)
lcd.vline(0,0,80,BLUE)
lcd.vline(159,0,80,BLUE)
lcd.display()
utime.sleep(3)
# Text sizes
lcd.fill(colour(0,0,0))
lcd.rect(0,0,159,79,colour(0,0,255)) # Blue Frame - method 2
centrestring("Raspberry Pi Pico",5,1,0,0,colour(255,0,0))
centrestring("Tonygo2",23,2,0,0,colour(255,255,0))
centrestring("Size 3",50,3,0,0,colour(255,0,255))
lcd.display()
utime.sleep(5)

# Gradients
lcd.fill(0)
centrestring("Gradients",30,2,0,0,colour(200,200,200))
lcd.display()
utime.sleep(3)
for x in range(160):
    c= colour(int(x*1.5),0,0)
    lcd.vline(x,0,80,c)
lcd.display()
utime.sleep(3)
for x in range(160):
    c= colour(0,int(x*1.5),0)
    lcd.vline(x,0,80,c)
lcd.display()
utime.sleep(3)
for x in range(160):
    c= colour(0,0,int(x*1.5))
    lcd.vline(159-x,0,80,c)
lcd.display()
utime.sleep(3)
for x in range(160): # Dual colour gradient
    c= colour(int((159-x)*1.5),0,int(x*1.5))
    lcd.vline(159-x,0,80,c)
centrestring("Dual",20,2,0,0,0)
centrestring("Gradient",38,2,0,0,0)
lcd.display()
utime.sleep(3)

# Random circles & Rings
lcd.fill(0)
lcd.display()
centrestring("Circles",15,3,0,0,colour(200,200,0))
centrestring("& Rings",40,3,0,0,colour(200,200,0))
lcd.display()
utime.sleep(2)

lcd.fill(0)
lcd.display()
for cc in range(200):
    c = colour(random.randint(40,255),random.randint(40,255),random.randint(40,255))
    ring(random.randint(5,155),random.randint(5,75),random.randint(5,20),c)
    lcd.display()
utime.sleep(2)
lcd.fill(0)
lcd.display()
y = 50
x = 20
for i in range(40):
    c = colour(random.randint(40,255),random.randint(40,255),random.randint(40,255))
    xx = 10+i*4
    yy = y-random.randint(0,20)
    rr = 40-i
    ring(xx,yy,rr,c)
    lcd.display()
utime.sleep(2)

lcd.fill(0)
lcd.display() 
c = colour(255,0,0)
circle(80,40,40,c)
lcd.display()
utime.sleep(1)
for cc in range(200):
    c = colour(random.randint(40,255),random.randint(40,255),random.randint(40,255))
    circle(random.randint(5,155),random.randint(5,75),random.randint(5,20),c)
    lcd.display()
utime.sleep(2)

lcd.fill(0)
lcd.display()
y = 50
x = 20
for i in range(40):
    c = colour(random.randint(40,255),random.randint(40,255),random.randint(40,255))
    xx = 10+i*4
    yy = y-random.randint(0,20)
    rr = 40-i
    circle(xx,yy,rr,c)
    lcd.display()
utime.sleep(2)
lcd.fill(0)
lcd.display() 

graphs() # Sine & Cosine curves

bar()    # Joystick controlled bar graph

# Colour Mixing on screen with Joystick and buttons
# ======= Centre Press joystick to halt =======
centrestring("Colour Mixing",10,1,0,0,colour(250,180,0))
centrestring("Move Joystick",27,1,0,0,colour(200,200,0))
centrestring("& press buttons",37,1,0,0,colour(200,200,0))
centrestring("Centre Jstk Halts",60,1,0,0,colour(200,200,200))
lcd.display()
utime.sleep(4)
lcd.fill(0)
r = 130
g = 130
b = 130
top = 50
w = 40
h = 15
# Basic colours
lcd.fill_rect(8,top,w,h,colour(r,0,0))
lcd.fill_rect(60,top,w,h,colour(0,g,0))
lcd.fill_rect(112,top,w,h,colour(0,0,b))
# Mixed colour    
c = colour(r,g,b)
lcd.fill_rect(30,10,100,h,c)
lcd.fill_rect(0,top+20,160,10,0)
running = True
while running:    
    if (KEY_UP.value() == 0): # Read joystick
        r = r + 5
        if r >= 255:
            r = 255
    if (KEY_DOWN.value() == 0):
        r = r - 5
        if r <= 0:
            r = 0
    if (KEY_RIGHT.value() == 0):
        g = g + 5
        if g >= 255:
            g = 255
    if (KEY_LEFT.value() == 0):
        g = g - 5
        if g <= 0:
            g = 0       
    if (KEY_A.value() == 0): # Read buttons
        b = b + 5
        if b >= 255:
            b = 255
    if (KEY_B.value() == 0):
        b = b - 5
        if b <= 0:
            b = 0
    if (KEY_CTRL.value() == 0): # Centre press on joystick
        running = False         # Exit the loop 
    lcd.fill_rect(0,top+20,160,10,0)
    c = colour(150,150,150)
    printstring(str(r),18,top + 20,1,0,0,c)
    printstring(str(g),70,top + 20,1,0,0,c)
    printstring(str(b),122,top + 20,1,0,0,c)
    printstring(str(b),122,top + 20,1,0,0,c)
    lcd.fill_rect(8,top,w,h,colour(r,0,0))
    lcd.fill_rect(60,top,w,h,colour(0,g,0))
    lcd.fill_rect(112,top,w,h,colour(0,0,b))   
    c = colour(r,g,b)
    lcd.fill_rect(30,10,100,h,c)
    lcd.fill_rect(55,30,80,10,0)
    printstring(str(hex(c)),55,30,1,0,0,colour(150,150,150))
    lcd.display()
 
# Tidy up 
lcd.fill(0)
printstring("Halted",25,30,3,0,0,colour(200,200,0))
lcd.display()
utime.sleep(5)
lcd.fill(0)
lcd.display()


        
